@@ -2,11 +2,12 @@ import collections
 import logging
 import pathlib
 import warnings
-from typing import Dict, List, Optional, cast
+from typing import TYPE_CHECKING, Optional, cast
 
 import yaml
 
 from snowflake.ml._internal import env_utils
+from snowflake.ml._internal.exceptions import error_codes, exceptions
 from snowflake.ml.data import data_source
 from snowflake.ml.model import type_hints
 from snowflake.ml.model._model_composer.model_manifest import model_manifest_schema
@@ -22,6 +23,9 @@ from snowflake.ml.model._packager.model_meta import (
 )
 from snowflake.ml.model._packager.model_runtime import model_runtime
 
+if TYPE_CHECKING:
+    from snowflake.ml.experiment._experiment_info import ExperimentInfo
+
 logger = logging.getLogger(__name__)
 
 
@@ -36,7 +40,6 @@ class ModelManifest:
     """
 
     MANIFEST_FILE_REL_PATH = "MANIFEST.yml"
-    _ENABLE_USER_FILES = False
     _DEFAULT_RUNTIME_NAME = "python_runtime"
 
     def __init__(self, workspace_path: pathlib.Path) -> None:
@@ -46,25 +49,53 @@ class ModelManifest:
         self,
         model_meta: model_meta_api.ModelMetadata,
         model_rel_path: pathlib.PurePosixPath,
-        user_files: Optional[Dict[str, List[str]]] = None,
+        user_files: Optional[dict[str, list[str]]] = None,
         options: Optional[type_hints.ModelSaveOption] = None,
-        data_sources: Optional[List[data_source.DataSource]] = None,
-        target_platforms: Optional[List[type_hints.TargetPlatform]] = None,
+        data_sources: Optional[list[data_source.DataSource]] = None,
+        experiment_info: Optional["ExperimentInfo"] = None,
+        target_platforms: Optional[list[type_hints.TargetPlatform]] = None,
     ) -> None:
         if options is None:
             options = {}
 
+        has_pip_requirements = len(model_meta.env.pip_requirements) > 0
+        only_spcs = (
+            target_platforms
+            and len(target_platforms) == 1
+            and target_platforms[0] == type_hints.TargetPlatform.SNOWPARK_CONTAINER_SERVICES
+        )
+
         if "relax_version" not in options:
-            warnings.warn(
-                (
-                    "`relax_version` is not set and therefore defaulted to True. Dependency version constraints relaxed"
-                    " from ==x.y.z to >=x.y, <(x+1). To use specific dependency versions for compatibility, "
-                    "reproducibility, etc., set `options={'relax_version': False}` when logging the model."
-                ),
-                category=UserWarning,
-                stacklevel=2,
-            )
-        relax_version = options.get("relax_version", True)
+            if has_pip_requirements or only_spcs:
+                logger.info(
+                    "Setting `relax_version=False` as this model will run in Snowpark Container Services "
+                    "or in Warehouse with a specified artifact_repository_map where exact version "
+                    " specifications will be honored."
+                )
+                relax_version = False
+            else:
+                warnings.warn(
+                    (
+                        "`relax_version` is not set and therefore defaulted to True. Dependency version constraints"
+                        " relaxed from ==x.y.z to >=x.y, <(x+1). To use specific dependency versions for compatibility,"
+                        " reproducibility, etc., set `options={'relax_version': False}` when logging the model."
+                    ),
+                    category=UserWarning,
+                    stacklevel=2,
+                )
+                relax_version = True
+            options["relax_version"] = relax_version
+        else:
+            relax_version = options.get("relax_version", True)
+            if relax_version and (has_pip_requirements or only_spcs):
+                raise exceptions.SnowflakeMLException(
+                    error_code=error_codes.INVALID_ARGUMENT,
+                    original_exception=ValueError(
+                        "Setting `relax_version=True` is only allowed for models to be run in Warehouse with "
+                        "Snowflake Conda Channel dependencies. It cannot be used with pip requirements or when "
+                        "targeting only Snowpark Container Services."
+                    ),
+                )
 
         runtime_to_use = model_runtime.ModelRuntime(
             name=self._DEFAULT_RUNTIME_NAME,
@@ -79,12 +110,13 @@ class ModelManifest:
             logger.info(f"Conda dependencies: {runtime_to_use.runtime_env.conda_dependencies}")
             logger.info(f"Pip requirements: {runtime_to_use.runtime_env.pip_requirements}")
             logger.info(f"artifact_repository_map: {runtime_to_use.runtime_env.artifact_repository_map}")
+            logger.info(f"resource_constraint: {runtime_to_use.runtime_env.resource_constraint}")
         runtime_dict = runtime_to_use.save(
             self.workspace_path, default_channel_override=env_utils.SNOWFLAKE_CONDA_CHANNEL_URL
         )
 
         self.function_generator = function_generator.FunctionGenerator(model_dir_rel_path=model_rel_path)
-        self.methods: List[model_method.ModelMethod] = []
+        self.methods: list[model_method.ModelMethod] = []
 
         for target_method in model_meta.signatures.keys():
             method = model_method.ModelMethod(
@@ -101,7 +133,7 @@ class ModelManifest:
 
             self.methods.append(method)
 
-        self.user_files: List[model_user_file.ModelUserFile] = []
+        self.user_files: list[model_user_file.ModelUserFile] = []
 
         if user_files is not None:
             for subdirectory, paths in user_files.items():
@@ -128,16 +160,19 @@ class ModelManifest:
         if model_meta.env.artifact_repository_map:
             dependencies["artifact_repository_map"] = runtime_dict["dependencies"]["artifact_repository_map"]
 
+        runtime = model_manifest_schema.ModelRuntimeDict(
+            language="PYTHON",
+            version=runtime_to_use.runtime_env.python_version,
+            imports=runtime_dict["imports"],
+            dependencies=dependencies,
+        )
+
+        if runtime_dict["resource_constraint"]:
+            runtime["resource_constraint"] = runtime_dict["resource_constraint"]
+
         manifest_dict = model_manifest_schema.ModelManifestDict(
             manifest_version=model_manifest_schema.MODEL_MANIFEST_VERSION,
-            runtimes={
-                self._DEFAULT_RUNTIME_NAME: model_manifest_schema.ModelRuntimeDict(
-                    language="PYTHON",
-                    version=runtime_to_use.runtime_env.python_version,
-                    imports=runtime_dict["imports"],
-                    dependencies=dependencies,
-                )
-            },
+            runtimes={self._DEFAULT_RUNTIME_NAME: runtime},
             methods=[
                 method.save(
                     self.workspace_path,
@@ -149,10 +184,10 @@ class ModelManifest:
             ],
         )
 
-        if self._ENABLE_USER_FILES:
+        if self.user_files:
             manifest_dict["user_files"] = [user_file.save(self.workspace_path) for user_file in self.user_files]
 
-        lineage_sources = self._extract_lineage_info(data_sources)
+        lineage_sources = self._extract_lineage_info(data_sources, experiment_info)
         if lineage_sources:
             manifest_dict["lineage_sources"] = lineage_sources
 
@@ -179,8 +214,10 @@ class ModelManifest:
         return res
 
     def _extract_lineage_info(
-        self, data_sources: Optional[List[data_source.DataSource]]
-    ) -> List[model_manifest_schema.LineageSourceDict]:
+        self,
+        data_sources: Optional[list[data_source.DataSource]],
+        experiment_info: Optional["ExperimentInfo"],
+    ) -> list[model_manifest_schema.LineageSourceDict]:
         result = []
         if data_sources:
             for source in data_sources:
@@ -198,4 +235,12 @@ class ModelManifest:
                             type=model_manifest_schema.LineageSourceTypes.QUERY.value, entity=source.sql
                         )
                     )
+        if experiment_info:
+            result.append(
+                model_manifest_schema.LineageSourceDict(
+                    type=model_manifest_schema.LineageSourceTypes.EXPERIMENT.value,
+                    entity=experiment_info.fully_qualified_name,
+                    version=experiment_info.run_name,
+                )
+            )
         return result
